/*
 * FILE: test_renewssp.c
 *
 * Comment: A program to test the value of the reference canary when a
 *          fork() is called. It shows the value of the canary at
 *          differen moments during the execution.
 *
 *          Examples of execution:
 *          $ ./test_renewssp 
 *          Canary before fork():           0x7ea6282dd5059d00
 *          Reference-canary of the parent: 0x7ea6282dd5059d00
 *          Reference-canary of the child:  0x7ea6282dd5059d00
 *
 *          $ LD_PRELOAD=./librenewssp.so  ./test_renewssp
 *          Canary before fork():           0x1a950c992d1f4400
 *          Reference-canary of the parent: 0x1a950c992d1f4400
 *          Reference-canary of the child:  0xb18b97ee897c2b00
 *
 * Author: Hector Marco <hecmargi@upv.es>
 *         Ismael Ripoll <iripoll@disca.upv.es>
 *
 * Date:   February 2014
 * 
 * Web:    http://renewssp.com    and    http://cybersecurity.upv.es
 *
 * (C) Copyright, 2014  Hector Marco and Ismael Ripoll
 * 
 * This Software is provided to you solely for testing and
 * evaluation. You may not use it for any other purpose.
 *
 * The software is provided "as is", without warranty of any kind,
 * express or implied, including but not limited to the warranties of
 * merchantability, fitness for a particular purpose and
 * noninfringement. In no event shall the authors or copyright holders
 * be liable for any claim, damages or other liability, whether in an
 * action of contract, tort or otherwise, arising from, out of or in
 * connection with the software or the use or other dealings in the
 * software.
 *
 */ 

#include <stdio.h>
#include <unistd.h>
#include <inttypes.h>

#ifdef __i386__
#define GET_CANARY(x) \
   asm volatile ("mov %%gs:0x14, %0" : "=r" (x));
#define FORMAT   "08x"
#elif defined __x86_64__
#define GET_CANARY(x) \
   asm volatile ("mov %%fs:0x28, %0" : "=r" (x));
#define FORMAT   "016lx"
#endif

uintptr_t getCanary(void){
   uintptr_t x;
   GET_CANARY(x);
   return x;
}
void printCanary(void){
   printf("0x%" FORMAT "\n", getCanary());
}

int main(void){

   pid_t pid;

   printf("Canary before fork():           ");
   printCanary();

   pid = fork();

   if(pid < 0){
      printf("fork() failed\n");
      return -1;
   }
   if(pid == 0){
      printf("Reference-canary of the child:  ");
      printCanary();
   }
   else {
      printf("Reference-canary of the parent: ");
      printCanary();
   }

   return 0;
}

